package net.knarcraft.minecraftserverlauncher.server.servertypes;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import net.knarcraft.minecraftserverlauncher.server.ServerVersionContainer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Paths;

import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.downloadFile;
import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.readFile;

/**
 * This class represents the regular vanilla Minecraft server type
 */
public class Vanilla extends AbstractServerType {

    private final String versionURL;

    /**
     * Instantiates a vanilla server type
     *
     * @param typeName    <p>The name of this server type to display</p>
     * @param isProxy     <p>Whether this server type is a proxy server</p>
     * @param versions    <p>Available versions for this server type</p>
     * @param versionURL  <p>The URL used for downloading the version document</p>
     * @param downloadURL <p>The URL used for downloading the new file</p>
     */
    public Vanilla(String typeName, boolean isProxy, String[] versions, String versionURL, String downloadURL) {
        super(typeName, isProxy, versions, downloadURL);
        this.versionURL = versionURL;
    }

    @Override
    public boolean downloadJar(String folder, String version) throws IOException {
        ServerVersionContainer versionContainer = ServerVersionContainer.getInstance();
        String file = this.getName() + version + ".jar";
        File filePath = new File(folder + file);
        if (version.equals("Latest") || version.equals("Snapshot")) {
            String releaseType = version.equals("Latest") ? "release" : "snapshot";
            String lastVersion = version.equals("Latest") ? versionContainer.getVanillaVersion() :
                    versionContainer.getSnapshotVersion();
            return downloadLatestJar(filePath, releaseType, lastVersion, versionContainer);
        } else {
            String downloadURL = getVanillaDownloadURL(getServerFileVersionURL(version));
            return filePath.isFile() || downloadFile(downloadURL, Paths.get(filePath.toURI()));
        }
    }

    /**
     * Downloads the latest .jar file found if necessary
     *
     * @param filePath    <p>The path of the jar file to download</p>
     * @param releaseType <p>The release type used for downloading</p>
     * @param lastVersion <p>The last server version found</p>
     * @param versionContainer <p>The version container to use</p>
     * @return <p>True if the jar exists and is the latest version or was downloaded</p>
     * @throws IOException <p>If the .jar cannot be downloaded</p>
     */
    private boolean downloadLatestJar(File filePath, String releaseType, String lastVersion,
                                      ServerVersionContainer versionContainer) throws IOException {
        String[] latestData = getLatestFile(releaseType);
        String latest = latestData[0];
        String jarFile = latestData[1];
        //The file is already the newest version
        if (filePath.isFile() && latest.equals(lastVersion)) {
            return true;
        }
        //The new jar file could not be downloaded
        if (!downloadFile(jarFile, Paths.get(filePath.toURI()))) {
            return false;
        }
        if (releaseType.equals("release")) {
            versionContainer.setVanillaVersion(latest);
        } else {
            versionContainer.setSnapshotVersion(latest);
        }
        return true;
    }

    /**
     * Gets the URL to the .jar file for the newest version
     *
     * @param releaseType <p>The type of release to read latest version from</p>
     * @return <p>An array containing the latest version and a link to its file</p>
     * @throws IOException <p>If the remote resource cannot be readFromServer</p>
     */
    private String[] getLatestFile(String releaseType) throws IOException {
        String versionText = readFile(versionURL);
        JsonObject jsonObject = new JsonParser().parse(versionText).getAsJsonObject();
        String latest = jsonObject.getAsJsonObject("latest").get(releaseType).getAsString();
        String versionURL = getServerFileVersionURL(latest);
        String jarURL = getVanillaDownloadURL(versionURL);
        return new String[]{latest, jarURL};
    }

    /**
     * Gets the URL necessary for downloading a given minecraft .jar file
     *
     * @param versionURL <p>The URL to the version document describing the .jar file</p>
     * @return <p>The URL necessary do download the .jar file</p>
     * @throws IOException <p>If the remote resource cannot be readFromServer</p>
     */
    private String getVanillaDownloadURL(String versionURL) throws IOException {
        String versionText = readFile(versionURL);
        JsonObject jsonObject = new JsonParser().parse(versionText).getAsJsonObject();
        return jsonObject.getAsJsonObject("downloads").getAsJsonObject("server").get("url").getAsString();
    }

    /**
     * Gets the version URL from the Minecraft vanilla version document
     *
     * @param targetVersion <p>The version to download</p>
     * @return <p>The URL to the file</p>
     * @throws IOException <p>If the file cannot be downloaded</p>
     */
    private String getServerFileVersionURL(String targetVersion) throws IOException {
        String versionText = readFile(versionURL);
        JsonObject jsonObject = new JsonParser().parse(versionText).getAsJsonObject();
        JsonArray availableVersions = jsonObject.getAsJsonArray("versions");
        for (JsonElement availableVersion : availableVersions) {
            JsonObject versionObject = availableVersion.getAsJsonObject();
            if (!versionObject.get("id").getAsString().equals(targetVersion)) {
                continue;
            }
            return versionObject.get("url").getAsString();
        }
        throw new FileNotFoundException("Unable to find the requested Minecraft vanilla .jar file.");
    }

}
