package net.knarcraft.minecraftserverlauncher.userinterface;

import net.knarcraft.minecraftserverlauncher.Main;

import javax.swing.*;
import javax.swing.text.DefaultCaret;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import static javax.swing.text.DefaultCaret.ALWAYS_UPDATE;

/**
 * Acts as a single writable/readable tab
 *
 * <p>Has a box for user input, and a textArea for server output.</p>
 *
 * @author Kristian Knarvik <kristian.knarvik@knett.no>
 * @version 1.0.0
 * @since 1.0.0
 */
public class Console extends KeyAdapter implements ActionListener, KeyListener {
    private final JTextField textInput;
    private final JTextArea textOutput;
    private final String name;
    private final JPanel panel;
    private final ArrayList<String> commands = new ArrayList<>();
    private int commandIndex;
    private final int maxConsoleLines = 1000;

    /**
     * Instantiates a new console
     *
     * @param tab <p>The tabbed pane used for displaying the console</p>
     * @param name <p>The name of the console tab</p>
     */
    Console(JTabbedPane tab, String name) {
        this.name = name;
        panel = new JPanel();
        tab.addTab(name, null, panel, null);
        panel.setLayout(new BorderLayout(0, 0));
        textInput = new JTextField();
        panel.add(textInput, BorderLayout.SOUTH);
        textInput.setColumns(10);
        textInput.addActionListener(this);
        textOutput = new JTextArea();
        JScrollPane scroll = new JScrollPane(textOutput);
        panel.add(scroll, BorderLayout.CENTER);
        textOutput.setEditable(false);
        DefaultCaret caret = (DefaultCaret) textOutput.getCaret();
        caret.setUpdatePolicy(ALWAYS_UPDATE);
        textOutput.setLineWrap(true);
        textInput.addKeyListener(this);
    }

    /**
     * Gets the JPanel this console is drawn on
     *
     * @return <p>The JPanel this console is drawn on</p>
     */
    public JPanel getPanel() {
        return this.panel;
    }

    /**
     * Prints a string to the console's output
     *
     * @param text <p>The string to print</p>
     */
    public void output(String text) {
        int outputLines = this.textOutput.getLineCount();
        if (outputLines > maxConsoleLines) {
            truncateConsole(outputLines);
        }
        this.textOutput.setText(this.textOutput.getText() + "\n" + text);
    }

    /**
     * Truncates the first 50 lines if the console output has reached the max limit
     * @param outputLines <p>The currently readable lines in the console output field.</p>
     */
    private void truncateConsole(int outputLines) {
        String oldText = this.textOutput.getText();
        String[] oldTextList = oldText.split("\n");
        for (int i = 0; i < outputLines - maxConsoleLines + 50; i++) {
            oldTextList[i] = "";
        }
        StringBuilder newTextBuilder = new StringBuilder();
        for (String line : oldTextList) {
            if (!line.equals("")) {
                newTextBuilder.append(line);
            }
        }
        this.textOutput.setText(newTextBuilder.toString());
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        //Sends the command from the input to the server with the same name.
        if (e.getSource() == textInput) {
            java.lang.String text = textInput.getText();
            Main.getController().getCurrentProfile().sendCommand(this.name, text);
            commands.add(text);
            if (commands.size() > 25) {
                commands.remove(0);
            }
            commandIndex = commands.size();
            textInput.setText("");
        }
    }

    @Override
    public void keyPressed(KeyEvent e) {
        //Cycles through previously used commands
        if (e.getKeyCode() == KeyEvent.VK_UP) {
            showPreviousCommand();
        } else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
            showNextCommand();
        }
    }

    /**
     * Shows the previously executed command in the input field
     *
     * <p>Shows the previously executed command if a command was just executed.
     * Shows the command executed earlier if already showing a previously executed command.</p>
     */
    private void showPreviousCommand() {
        if (commands.size() > 0 && commandIndex > 0) {
            textInput.setText(commands.get(--commandIndex));
        }
    }

    /**
     * Shows the next previously executed command or clears the input field
     *
     * <p>Shows the next previously executed command if such a command exists.
     * Clears the input field if no next used command exists.</p>
     */
    private void showNextCommand() {
        if (commands.size() > 0) {
            if (commandIndex == commands.size() - 1) {
                commandIndex++;
                textInput.setText("");
            } else if (commandIndex >= 0 && commandIndex <= commands.size() - 1) {
                textInput.setText(commands.get(++commandIndex));
            }
        }
    }
}
